@echo off
python rml2pdf.pyc doc\rml_user_guide.rml doc\example_1.rml doc\example_2.rml doc\example_3.rml doc\example_4.rml doc\example_5.rml doc\example_6.rml doc\example_7a.rml doc\example_7b.rml doc\example_8.rml doc\example_9.rml doc\example_10.rml doc\example_11.rml doc\example_12.rml
python rml2pdf.pyc test\test_001_hello.rml test\test_002_paras.rml test\test_003_frames.rml test\test_004_templates.rml test\test_005_fonts.rml test\test_006_barcodes.rml test\test_007_forms.rml test\test_008_tables.rml test\test_02_001_newdtd.rml test\test_02_002_newdtd.rml test\test_02_003_newdtd.rml
python rml2pdf.pyc demos\heart.rml demos\snow.rml demos\sample.rml
